#!/usr/bin/perl
# ------------------------------------------------------------------------------
#  ident "@(#)men_jumpstart.pl 1.16 02/04/25 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}

  my $SOLARIS_DIST ;

#-------------------------------------------------------------------------------
#
# SVM installation and configuration
#
#-------------------------------------------------------------------------------

sub svm_install {

  my $mountpoint = "/a/mnt_tmp" ;
  my $server_ip  ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${SERVER_NODE}";
  
  # installation of package: there are not locate on the standard directory

   open (FINISH, ">>$JUMPSTART/$CLUSTER/$NODE_ID/finish.sh") || 
     &error("Unable to update \"$JUMPSTART/$CLUSTER/$NODE_ID/finish.sh\" file");

   print FINISH <<"FEO";

$RMOUNT -v
$RMKDIR -p $mountpoint

# mount distribution
$RMOUNT $server_ip:/$SOLARIS_DIR $mountpoint
$RMOUNT -v

# create admin file
$RECHO "mail=" > /a/tmp/admin
$RECHO "instance=overwrite" >> /a/tmp/admin
$RECHO "partial=nocheck" >> /a/tmp/admin
$RECHO "runlevel=quit" >> /a/tmp/admin
$RECHO "idepend=nocheck" >> /a/tmp/admin
$RECHO "rdepend=nocheck" >> /a/tmp/admin
$RECHO "space=quit" >> /a/tmp/admin
$RECHO "setuid=nocheck" >> /a/tmp/admin
$RECHO "conflict=nocheck" >> /a/tmp/admin
$RECHO "action=nocheck" >> /a/tmp/admin
$RECHO "basedir=default" >> /a/tmp/admin

# install packages
$RPKGADD -d $mountpoint/$PACKAGE_SUBDIR -R /a -a /a/tmp/admin $LOCAL_PACKAGES
# umount distribution
$RUMOUNT $mountpoint
$RRMDIR $mountpoint

FEO

  close(FINISH);

}

#-------------------------------------------------------------------------------
#
# Create jumpstart environment
#
#-------------------------------------------------------------------------------
sub jumpstart_create {

   my ($cluster,$subnet0,$subnet1,$proto_node_id,$server_node) = @_;
   my $proto_name =build_node_name($proto_node_id, "");
   my $server_name=build_server_name();
   my $proto_ip   ="${subnet0}.${cluster}.${subnet1}.${proto_node_id}";
   my $server_ip  ="${subnet0}.${cluster}.${subnet1}.${server_node}";
   my $ether = $NODE_LIST[getnode($proto_node_id)]{eth0};
   
   #
   # Controle the host configuration
   #

   #
   # Test the  jumpstart directory tree, create it if it does not exist
   #
   &control_dir_rxw_or_create ( "$JUMPSTART" );
   &control_dir_rxw_or_create ( "$JUMPSTART/$cluster" );
   &control_dir_rxw_or_create ( "$JUMPSTART/$cluster/$proto_node_id" );

   #
   # Create Jumpstart config files
   #
   &create_sysidcfg ($cluster,$subnet0,$subnet1,$proto_node_id,$server_node);
   &create_profile ($cluster, $proto_node_id);
   &create_finish($cluster,$subnet0,$subnet1,$proto_node_id,$server_node);
   &create_and_test_rules($cluster,$subnet0,$subnet1,$proto_node_id,$server_node);   
   
   # 
   # Control and modify /etc/hosts
   #   
   &add_host( $proto_ip, $proto_name  );
   &add_host( $server_ip,$server_name );
   
   # 
   # Control end modify /etc/ethers
   #
   &add_ether ( $ether, $proto_name );
    
   #
   # Purge preexistant boot client, ignore errors silently
   #
   &exec_cmd_ignore ("$SOLARIS_DIST/Tools/rm_install_client $proto_name > /dev/null 2>&1") ;
   
   #
   # Share: 
   #        Solaris distribution
   #        Jumpstart environment
   #        Packages repository
   #   
   &exec_cmd ( "$SHARE -F nfs -o ro,anon=0 $SOLARIS_DIR" ) || 
   &error ( "Unable to NFS share $SOLARIS_DIR" );   
   
   &exec_cmd ( "$SHARE -F nfs -o ro,anon=0 $JUMPSTART" ) ||
   &error ( "Unable to NFS share $JUMPSTART" );
      
   #
   # Add boot client
   #
   $cmd="$SOLARIS_DIST/Tools/add_install_client";
   $cmd .= " -i \"$proto_ip\"";
   $cmd .= " -e \"$ether\"";
   $cmd .= " -s \"$server_ip:$SOLARIS_DIR\"";
   $cmd .= " -c \"$server_ip:${JUMPSTART}/${cluster}/${proto_node_id}\"";
   $cmd .= " -p \"$server_ip:${JUMPSTART}/${cluster}/${proto_node_id}\"";
   $cmd .= " -n :none";
   $cmd .= " $proto_name $ARCH";   
   &exec_cmd ($cmd) || &error ("Execution of \"$cmd\" failed"); 
   
   #
   # Remplace install server name by server IP address in /etc/bootparams
   #
   $cmd="/usr/bin/perl -p -i.bak";
   $cmd .= " -e 'if ( /^$proto_name/ ) {";
   $cmd .= " s/^$proto_name\\s+[^:]+:/$proto_name root=$server_ip:/;";
   $cmd .= " }'";
   $cmd .= " /etc/bootparams";
   &exec_cmd ($cmd) || &error("Can't modify \"/etc/bootparams\" file");
      
}


#-------------------------------------------------------------------------------
#
# Managemnt of /etc/hosts and /etc/ethers files
#
#-------------------------------------------------------------------------------
sub add_host {
   my ( $address, $name ) = @_ ;
   my $count=0;
   my $ip;
   my $host;
   my $garbage;
 
   $count = 0;   
     
   open (HOSTS, "/etc/hosts" ) || &error("Unable to open /etc/hosts");
   while ( <HOSTS> ) {
      
      chop();
      s/#.*//;
      ($ip, $host, $garbage) = split (/ /, $_) ;
      next unless (&defined($ip));
      next unless (&defined($host));
            
      if ( "$ip" eq "$address" ) {
	 &error("Ip \"$address\" does not match \"$name\" as first hostname in /etc/hosts") 
	 unless ( $host eq $name );
         $count++;
	 last;
      }
       	        
   }
   close (HOSTS);
   
   unless ( $count ) {
      open (HOSTS, ">>/etc/hosts" ) || &error("Unable to update /etc/hosts");
      print HOSTS "$address $name\n";
      close (HOSTS);
   }
   
}

sub add_ether {
   
   my ( $address, $name ) = @_ ;
   my $count=0;
   my $ether;
   my $host;
      
   if ( -f "/etc/ethers" ) {
      open (ETHERS, "/etc/ethers" ) || &error("Unable to open /etc/ethers");
      while ( <ETHERS> ) {
      
         chop();
         s/#.*//;
         ($ether, $host) = split (/ /, $_) ;
         next unless (&defined($ether));
         next unless (&defined($host));
      
         if ( &ethermatch ("$ether","$address") ) {  
	    &error("MAC \"$address\" does not match \"$name\" as hostname in /etc/ethers")
	    unless ( $host eq $name );
            $count++;
	    last;
         }
       	        
      }
      close (ETHERS);
   }
   
   unless ( $count ) {
      open (ETHERS, ">>/etc/ethers" ) || &error("Unable to update /etc/ethers");
      print ETHERS "$address $name\n";
      close (ETHERS);

      &exec_cmd("$PKILL in.rarpd ; /usr/sbin/in.rarpd -a") ||
   	&error("Unable to kill and restart rarpd daemon file");

   }

}


sub duplicated_ether {
   
   my ( $address ) = @_ ;
   my $count=0;
   my $ether;
   my $host;
         
   if ( -f "/etc/ethers" ) {
      open (ETHERS, "/etc/ethers" ) || &error("Unable to open /etc/ethers");
      while ( <ETHERS> ) {      
         chop();
         s/#.*//;
         ($ether, $host) = split (/ /, $_) ;
         next unless (&defined($ether));
         next unless (&defined($host));      
         $count++ if ( &ethermatch($ether,$address) );         	        
      }
      close (ETHERS);
   }

   return ( $count > 1 );
      
}

sub ethermatch {

   my ( $addr1, $addr2 ) = @_;
   
   return 0 unless (&defined($addr1));
   return 0 unless (&defined($addr2));
   
   @a = split ( /:/, $addr1 );
   @b = split ( /:/, $addr2 );
   
   for ( $i=0; $i < 6; $i++ ) {
      return 0 unless (hex($a[$i]) == hex($b[$i])); 
   }
   
   return 1;
      
}

#-------------------------------------------------------------------------------
#
# update the /etc/netmasks with the cluster network mask
#
#-------------------------------------------------------------------------------

sub add_netmask {
   
  my ( $network, $mask ) = @_ ;
  my $count=0;
  my $read_network;
  my $read_mask;
  
  if ( -f "/etc/netmasks" ) {
    open (NETMASKS, "/etc/netmasks" ) || &error("Unable to open /etc/netmasks");
    while ( <NETMASKS> ) {
      
      chop();
      s/#.*//;
      ($read_network, $read_mask) = split (/[\s\t]+/, $_) ;
      next unless (&defined($read_network));
      next unless (&defined($read_mask));
      
      if ( "$network" eq "$read_network" ) {
	if ("$mask" ne "$read_mask") {
	  &error("Mask \"$read_mask\" does not match \"$mask\" for network $network") ;
	}
	$count++;
	last;
      }
      
      
    }
    close (NETMASKS);
  }
  
  unless ( $count ) {
    open (NETMASKS, ">>/etc/netmasks" ) || &error("Unable to update /etc/netmasks");
    print NETMASKS "$network\t$mask\n";
    close (NETMASKS);
  }
  
}

#-------------------------------------------------------------------------------
#
# Create Sysidcfg file
#
#-------------------------------------------------------------------------------
sub create_sysidcfg {

   my ($cluster,$subnet0,$subnet1,$proto_node_id,$server_node) = @_;

   my $proto_name = build_node_name($proto_node_id, "");
   my $timezone = "" ;

   # get the local information of the installation server
   open(FILE, "</etc/default/init") ;
   while (<FILE>)
     {
       chop();
       s/#.*//;
       ($var, $value) = split (/=/, $_) ;
       next unless ($var ne "") ;
       if ($var eq "TZ")
	 {
	   $timezone = $value ;
	 }
     }
   close(FILE) ;

   $timezone = "US/Central" unless ($timezone ne "") ;
    
   open (SYSIDCFG, ">$JUMPSTART/$cluster/$proto_node_id/sysidcfg") || 
   &error("Unable to create \"$JUMPSTART/$cluster/$proto_node_id/sysidcfg\" file");
   print SYSIDCFG <<"FEO";
name_service=NONE
system_locale=en_US
timezone=$timezone
timeserver=$subnet0.$cluster.$subnet1.$server_node
security_policy=NONE
terminal=vt100
network_interface=primary {protocol_ipv6=no
                           hostname=$proto_name
                           ip_address=$subnet0.$cluster.$subnet1.$proto_node_id
		      	   default_route=$subnet0.$cluster.$subnet1.$server_node
                           netmask=255.255.255.0}
root_password=mWlhY0O7LCRBc
FEO

   close(SYSIDCFG);

}

#-------------------------------------------------------------------------------
#
# Create jumpstart profile file
#
#-------------------------------------------------------------------------------
sub create_profile {
  my ($cluster, $node_id) = @_ ;

  my $profile = "$JUMPSTART/$cluster/$node_id/node.prof" ;

  my $slice_root ;
  my $idx;

  exec_cmd( "$CP $PROFILE $profile") ||
      &error("Unable to create $profile file");

  slice_search(mounting_point, $SLICE_ROOT, \$idx ) ;
  $slice_root= $SLICE_LIST[$idx]{name};

  open(FPROFILE, ">>$profile") ;

  print FPROFILE "boot_device $slice_root update\n" ;
  print FPROFILE "root_device $slice_root\n" ;

  for $idx (0..$#SLICE_LIST)
    {
      print FPROFILE "filesys $SLICE_LIST[$idx]{name} $SLICE_LIST[$idx]{size} $SLICE_LIST[$idx]{mounting_point} $SLICE_LIST[$idx]{option}\n"
    }
  
  #
  # generate a "usedisk" directive for each disk present in the configuration file
  #
  my %disk_array;
  for my $i (0..$#SLICE_LIST) {
    my $disk = substr($SLICE_LIST[$i]{name}, 0, 6);
    if (! exists $disk_array{$disk}) {
      print FPROFILE "usedisk $disk\n" ;
      $disk_array{$disk} = "";
    }
  }

  close FPROFILE ;

}

#------------------------------------------------------------------------------- 
#
# Create jumpstart rules file
#
#-------------------------------------------------------------------------------
sub create_and_test_rules {

   my ($cluster,$subnet0,$subnet1,$proto_node_id,$server_node) = @_;
  
   my $output = "";

   open (RULES, ">$JUMPSTART/$cluster/$proto_node_id/rules") || 
   &error("Unable to create \"JUMPSTART/$cluster/$proto_node_id/rules\" file");
   print RULES "hostaddress $subnet0.$cluster.$subnet1.$proto_node_id - node.prof finish.sh\n";
   close (RULES);

   open (CHECK,"cd $JUMPSTART/$cluster/$proto_node_id; $SOLARIS_DIST/Misc/jumpstart_sample/check |") || 
   &error ("Unable to check \"rules\" file"); 
      while ( <CHECK> ) {
         chop();
         &trace (1, $_ );
	 $output= "$output $_\n";
      }
   if (! close (CHECK)) {
     print_log($output);
     error("Error found while checking \"Rules\" file");
   }
}

#-------------------------------------------------------------------------------
#
# Create jumpstart finish.sh file
#
#-------------------------------------------------------------------------------
sub create_finish {

   my ($cluster,$subnet0,$subnet1,$proto_node_id,$server_node) = @_;
   my $install_ip = "$SUBNET0.$CLUSTER.$SUBNET1.$SERVER_NODE";
   
   open (FINISH, ">$JUMPSTART/$cluster/$proto_node_id/finish.sh") || 
   &error("Unable to create \"$JUMPSTART/$cluster/$proto_node_id/finish.sh\" file");
   print FINISH "#!/bin/sh\n\n";

   print FINISH <<"FEO";
#
# create /etc/notrouter
#
if [ ! -f /a/etc/notrouter ] ; then
/usr/bin/touch /a/etc/notrouter
fi

#
# modify /etc/default/login
#
/usr/bin/mv /a/etc/default/login /a/etc/default/login.orig
/usr/bin/chmod 644 /a/etc/default/login.orig
/usr/bin/sed '1,\$s/^CONSOLE/#CONSOLE/' /a/etc/default/login.orig > /a/etc/default/login
/usr/bin/chmod 444 /a/etc/default/login

#
# disable power management
#
if [ ! -f /a/noautoshutdown ] ; then
/usr/bin/touch /a/noautoshutdown
fi


#
# modify /.rhosts
#
if [ -f /a/.rhosts ] ; then
/usr/bin/cp /a/.rhosts /a/.rhosts.orig
fi
echo "$install_ip root" > /a/.rhosts
/usr/bin/chmod 444 /a/.rhosts

#
# set boot PROM params
#
/usr/sbin/eeprom local-mac-address?=true
/usr/sbin/eeprom auto-boot?=true


FEO

   close (FINISH);

}

#-------------------------------------------------------------------------------
#
# determine NIC to configure
#
#-------------------------------------------------------------------------------

sub determine_iface
  {
    my ($interface_to_configure) = @_ ;

    my $exist_physical = FALSE ;
    my $current_interface ;
    my $current_ip ;
    my $current_netmask ;
    my $current_broadcast ;
    my $interface ;
    my $route_interface ;
    my $physical ;
    my $alias ;
    my $ip ;
    my $broadcast ;
    my $netmask ;
    my $nodeid ;
    my $network;
    my $gateway;
    my $flags;
    my $ref;
    my $use;
    my $found = FALSE ;
    
    error("You must specify real network interface name, not an alias (aka name:number)")
      if ( &defined($IFACE) && $IFACE =~ /:/ );
    
    #
    # look for an interface with a matching IP Address
    #
    open (IFCONFIG, "$IFCONFIG -a |") ||
      error ("Unable to perform \"$IFCONFIG -a\" command") ;
    
    while ( <IFCONFIG> ) {
      chop() ;
      
      if (m/([^\s\t]*): flags/) {
	$current_interface = $1 ;
	
	# if the physical address exists, keep the information
	if ($current_interface eq $IFACE) {
	  $exist_physical = TRUE ;
	}
	
      } elsif (m/inet ([^\s\t]*) netmask ([^\s\t]*) broadcast ([^\s\t]*)/) {
	$current_ip = $1 ;
	$current_netmask = $2 ;
	$current_broadcast = $3 ;
	$current_ip =~ m/([0-9]*).([0-9]*).([0-9]*).([0-9]*)/ ;

	# check if the IP is in the subnet
	if (($1 eq $SUBNET0) && ($2 eq $CLUSTER) && ($3 eq $SUBNET1)) {
	  if ($found eq TRUE) {
	    error("At least two interfaces for cluster access ($interface and $current_interface): may cause problems\n") ;
	  }
	  $found = TRUE ;
	  $interface = $current_interface ;
	  ($physical, $alias) = split(/:/, $interface) ;
	  $ip = $current_ip;
	  $netmask = $current_netmask;
	  $broadcast = $current_broadcast;
	  $nodeid = $4 ;
	}
      }
    }
    close(IFCONFIG) ;

    if ($found ne TRUE) {
      if ($exist_physical eq FALSE) {
	# use it because we can not create an alias
	$$interface_to_configure = $IFACE ;
      } else {
	# the physical exists and it's not for cluster access (because no cluster access found)
	# create an alias
	$$interface_to_configure = "$IFACE:$CLUSTER" ;
      }
      return;
    }


    # check if there is no conflict
    if ($physical ne $IFACE) {
      &error("Configuration of $IFACE requested but cluster sub-network $SUBNET0.$CLUSTER.$SUBNET1 already configured on another physical device (interface $interface)") ;
    }

    # cluster access found: reuse the same interface
    $$interface_to_configure = $interface;

    return;
}

#-------------------------------------------------------------------------------
#
# Configure NIC
#
#-------------------------------------------------------------------------------
sub configure_iface {
   my ($cluster,$subnet0,$subnet1,$proto_node_id,$server_node) = @_;

   my $interface_to_configure;
   
   determine_iface(\$interface_to_configure) ;

   print_subaction("configuring $interface_to_configure") ;

   add_netmask("$SUBNET0.$CLUSTER.$SUBNET1.0", "255.255.255.0");
   
   #
   # Unplumb and "re"plumb  alias corresponding to cluster/subnet
   # caution: ignore the error if it didn't exist
   #
   &exec_cmd_ignore ("$IFCONFIG $interface_to_configure unplumb > /dev/null 2>&1");
   &exec_cmd ("$IFCONFIG $interface_to_configure plumb")||
     &error("Unable to plumb \"$interface_to_configure\" network interface");
   
   #
   # Configure interface
   #
   &exec_cmd ("$IFCONFIG $interface_to_configure $subnet0.$cluster.$subnet1.$server_node netmask 255.255.255.0 broadcast $subnet0.$cluster.$subnet1.255 up -private") ||
     &error("Unable to configure \"$interface_to_configure\" network interface");

   #
   # as the interface has just been created or re-created, the rarpd may be not aware
   # that it exists
   # it needs to be relaunched in order to take into account the new interface.
   #
   &exec_cmd("$PKILL in.rarpd ; /usr/sbin/in.rarpd -a") ||
     &error("Unable to kill and restart rarpd daemon file");
 }

##################################################################################
#
#                               This is the main
#
##################################################################################

@required = [ NODE_ROLE ] ;
@optional = [ ROUTER, JUMPSTART, PROFILE, SOLARIS_DIR, PACKAGE_SUBDIR, LOCAL_PACKAGES] ;
init_and_read_arg($0, @required, @optional) ;

if ( $SOLARIS_INSTALL ne "NO" ) {
  &usage($0, @required, @optional) unless &defined($SOLARIS_DIR);
  &usage($0, @required, @optional) unless &defined($PROFILE);
  # /etc/notrouter 
  if ( &defined($ROUTER) ) {
    &error ("File /etc/notrouter must exist on this host") unless ( -f "/etc/notrouter" );
  }  
}

#--------------------------------------------------------------------------------#
#
# Use absolute pathname for directories
#
#--------------------------------------------------------------------------------#

if (  $SOLARIS_INSTALL ne "NO" ) {
   $JUMPSTART = "$EXECDIR"."/".$JUMPSTART unless ( $JUMPSTART =~ /^\// );
   $SOLARIS_DIR = "$EXECDIR"."/".$SOLARIS_DIR unless ( $SOLARIS_DIR =~ /^\// );
}


#--------------------------------------------------------------------------------#
#
# Test Solaris distribution directory
#
#--------------------------------------------------------------------------------#
if ($SOLARIS_INSTALL ne "NO") {
   &control_dir_is_readable ("$SOLARIS_DIR");
   $SOLARIS_DIST=$SOLARIS_DIR . "/Solaris_8" if ( -d "$SOLARIS_DIR/Solaris_8" );
   $SOLARIS_DIST=$SOLARIS_DIR . "/Solaris_9" if ( -d "$SOLARIS_DIR/Solaris_9" );   
   &error("Unable to find Solaris 8 or Solaris 9 distribution on specified media") unless (&defined($SOLARIS_DIST));
   &control_dir_is_readable ("$SOLARIS_DIST");
   &control_dir_is_readable ("$SOLARIS_DIST/Tools");
   &control_dir_is_readable ("$SOLARIS_DIST/Tools/Boot");
   &control_dir_is_readable ("$SOLARIS_DIST/Misc");
   &control_dir_is_readable ("$SOLARIS_DIST/Misc/jumpstart_sample");

   &control_file_is_readable ("$SOLARIS_DIST/Misc/jumpstart_sample/check");
   
   &control_file_is_readable ( $PROFILE );
   
   &control_file_is_executable("$SOLARIS_DIST/Tools/add_install_client");
   &control_file_is_executable("$SOLARIS_DIST/Tools/rm_install_client");
   
   &control_dir_is_readable ("$SOLARIS_DIST/Tools/Boot/sbin");
   &control_file_is_executable ("$SOLARIS_DIST/Tools/Boot/sbin/bpgetfile");
 
}

if ($SOLARIS_INSTALL ne "NO") {
  print_stage_node("Solaris installation");
} else {
  print_stage("Installation server configuration")
}

#--------------------------------------------------------------------------------#
#
#  Test and/or Configure Network interface
#
#--------------------------------------------------------------------------------#

if ($NODE_ID eq $MEN1_ID) {
  print_action("Configuring network interface on installation server") ;
  configure_iface($CLUSTER, $SUBNET0, $SUBNET1, $NODE_ID , $SERVER_NODE); 
}

#--------------------------------------------------------------------------------#
#
#  We want to perform a jumpstart install of a prototype machine before being
#  able to create a Flash archive.
#
#--------------------------------------------------------------------------------#
if ($SOLARIS_INSTALL ne "NO") {

  # Prepare a jumpstart environment
  print_action("Preparing the Jumpstart environment");
  jumpstart_create ($CLUSTER, $SUBNET0, $SUBNET1, $NODE_ID, $SERVER_NODE);
  if ($USE_SVM eq "YES") {
    svm_install();
  }

}   


# That's all folk!
exit(0);







