#!/usr/bin/perl
# ------------------------------------------------------------------------------
#   ident "@(#)men_rnfs.pl 1.31 02/04/11 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}
                  

#--------------------------------------------------------------
#
# integer division (if there is a remaining, take the following
# integer value).
#		    
#--------------------------------------------------------------

sub divrounded
  {
    my ($value, $mod) = @_ ;

    my $res = int(($value-1)  / $mod + 1) ;
    return $res;
  }

#------------------------------------------------------------------------------
# Management of SNDR changes in /etc/system,/usr/kernel/drv/rdc.conf
# /usr/kernel/drv/mc_rms.conf, /usr/opt/SUNWrdc/lib/rdc_link_up
# which have the format <entry>=<value>
#------------------------------------------------------------------------------

sub update_rdc_file {
  my ( $rdc_file, $update_entry_input, $update_value_input ) = @_ ;
  my $count=0;
  my $update_entry;
  my $update_value;
  my $garbage;
 
  my $count = 0;   
  
  open (RDCFILE, $rdc_file ) || &error("Unable to open $rdc_file");
  open (RDCFILE_TMP, ">${rdc_file}.tmp" ) || &error("Unable to open ${rdc_file}.tmp");
	
  while ( <RDCFILE> ) {
    
    chop();
    ($update_entry, $update_value, $garbage) = split (/=/, $_) ;
    
    if ( "$update_entry" eq "$update_entry_input" ) {
      print RDCFILE_TMP "${update_entry}=${update_value_input}\n";
      $count++;
    } else {
      print RDCFILE_TMP "$_\n";
    }
    next;
    
  }
  close (RDCFILE);
  close (RDCFILE_TMP);
  
  unless ( $count ) {
    open (RDCFILE_TMP, ">>${rdc_file}.tmp" ) || &error("Unable to update $rdc_file");
    print RDCFILE_TMP "$update_entry_input=$update_value_input\n";
    close (RDCFILE_TMP);
  }
  
  &exec_cmd ( "$MV ${rdc_file}.tmp ${rdc_file}") ||
    &error ( "Unable to move ${rdc_file}.tmp ${rdc_file}");
  
}

sub update_vfstab_ufs {
  my ( $vfstab_file, $update_rdsk_input, $update_mountatboot_input ) = @_ ;
  my $update_rdsk;
  my $update_mountatboot;
  my $garbage1;
  my $garbage2;
  my $garbage3;
  my $garbage4;
  my $garbage5;
  my $garbage6;

  open (VFSTAB, $vfstab_file ) || &error("Unable to open $vfstab_file");
  open (VFSTAB_TMP, ">${vfstab_file}.tmp" ) || &error("Unable to open ${vfstab_file}.tmp");
	
  while ( <VFSTAB> ) {
    
    chop();
    s/#.*//;
    ($garbage1, $update_rdsk, $garbage2, $garbage3, $garbage4, $update_mountatboot, $garbage6) = split (/[\s\t]+/, $_) ;
    next unless (&defined($update_rdsk));
    next unless (&defined($update_mountatboot));
    
    if ( "$update_rdsk" eq "$update_rdsk_input" ) {
      print VFSTAB_TMP "$garbage1\t$update_rdsk\t$garbage2\t$garbage3\t$garbage4\t$update_mountatboot_input\t$garbage6\n";
    } else {
      print VFSTAB_TMP "$garbage1\t$update_rdsk\t$garbage2\t$garbage3\t$garbage4\t$update_mountatboot\t$garbage6\n";
    }
    
  }
  close (VFSTAB);
  close (VFSTAB_TMP);
  
  &exec_cmd ( "$MV ${vfstab_file}.tmp ${vfstab_file}") ||
    &error ( "Unable to move ${vfstab_file}.tmp ${vfstab_file}");  
}

sub update_vfstab_nfs {
   my ( $vfstab_file, $update_nfs_input, $update_nfs_value_input ) = @_ ;
   my $update_nfs;
   my $update_nfs_value;
   my $garbage;
   my $count = 0;
      
   open (VFSTAB, $vfstab_file ) || &error("Unable to open $vfstab_file");
   open (VFSTAB_TMP, ">${vfstab_file}.tmp" ) || &error("Unable to open ${vfstab_file}.tmp");
	
   while ( <VFSTAB> ) {
     
     chop();
     s/#.*//;
     ($update_nfs, $garbage) = split (/[\s\t]+/, $_) ;
            
     if ( "$update_nfs" eq "$update_nfs_input" ) {
       print VFSTAB_TMP "$update_nfs\t$update_nfs_value_input\n";
       $count++;
     } else {
       print VFSTAB_TMP "$_\n";
     }
     next;       	        
   }
   close (VFSTAB);
   close (VFSTAB_TMP);
	
   unless ($count) {
     open (VFSTAB_TMP, ">>${vfstab_file}.tmp" ) || &error("Unable to update $vfstab_file");
     print VFSTAB_TMP "$update_nfs_input\t$update_nfs_value_input\n";
     close (VFSTAB_TMP);
   }
	
   &exec_cmd ( "$MV ${vfstab_file}.tmp ${vfstab_file}") ||
     &error ( "Unable to move ${vfstab_file}.tmp ${vfstab_file}");  
}


#-------------------------------------------------------------------------------
#
# Configure RNFS 
#
#-------------------------------------------------------------------------------
sub rnfs_configure {

   my ($proto_node_id,$server_node) = @_;

   my $proto_name = build_node_name($proto_node_id, "");
   my $proto_ip   ="${SUBNET0}.${CLUSTER}.${SUBNET_NIC0}.${proto_node_id}";
	
   my $server_ip  ="${SUBNET0}.${CLUSTER}.${SUBNET_NIC1}.${server_node}";
   
   my $master_ip  ="${SUBNET0}.${CLUSTER}.${SUBNET_MAIN}.${MASTER_ID}";
	
   my $local_system = "$WORKING_DIR/system_${proto_name}";
   my $remote_system = "/etc/system";
   my $RDC_RPC_TMOUT = "set rdc:rdc_rpc_tmout";
   my $RDC_RPC_TMOUT_VALUE = "8";
   
   my $local_rdcconf = "$WORKING_DIR/rdcconf_${proto_name}";
   my $remote_rdcconf = "/usr/kernel/drv/rdc.conf";
   my $RDC_BITMAP_MODE_ENTRY = "rdc_bitmap_mode";
   my $RDC_BITMAP_MODE_VALUE = "1;";
   
   my $local_rdc_link_up = "$WORKING_DIR/rdc_link_up_${proto_name}";
   my $remote_rdc_link_up = "/usr/opt/SUNWrdc/lib/rdc_link_up";
   my $INTERVAL_ENTRY = "typeset -r INTERVAL";
   my $INTERVAL_VALUE = "\'5\'";

   my $local_mc_rms = "$WORKING_DIR/mc_rms.conf_${proto_name}";
   my $remote_mc_rms = "/usr/kernel/drv/mc_rms.conf";
   my $SYSTEMID_ENTRY = "systemid";
   my $SYSTEMID_VALUE = "3";
	
   my $local_sv_cf = "$WORKING_DIR/sv.cf_${proto_name}";
   my $remote_sv_cf = "/etc/opt/SUNWspsv/sv.cf";
	
   my $local_crfs_conf = "$WORKING_DIR/crfs.conf_${proto_name}";
   my $remote_crfs_conf = "/etc/opt/SUNWcgha/crfs.conf";
	
   my $local_nhcrfs_conf = "$WORKING_DIR/nhcrfs.conf_${proto_name}";
   my $remote_nhcrfs_conf = "/etc/opt/SUNWcgha/nhcrfs.conf";

   my $local_crfs_share = "$WORKING_DIR/crfs.share_${proto_name}";
   my $remote_crfs_share = "/etc/opt/SUNWcgha/crfs.share";
   
   my $local_vfstab = "$WORKING_DIR/vfstab_${proto_name}";
   my $remote_vfstab = "/etc/vfstab";
	
   my $i = 0;
	

   #
   # retrieve proto_node /usr/kernel/drv/rdc.conf file
   #
   &exec_cmd ( "$RCP $proto_ip:${remote_rdcconf} $local_rdcconf") ||
     &error ( "Unable to retrieve proto_node:$remote_rdcconf in $local_rdcconf");
	
   #
   # modify locally proto_node /usr/kernel/drv/rdc.conf file
   #
   &update_rdc_file( $local_rdcconf, $RDC_BITMAP_MODE_ENTRY, $RDC_BITMAP_MODE_VALUE);

   #
   # putback modified proto_node /usr/kernel/drv/rdc.conf file
   #
   &exec_cmd( "$RCP $local_rdcconf $proto_ip:${remote_rdcconf}" ) ||
     &error ( "Unable to putback $local_rdcconf on proto_node:${remote_rdcconf}") ;

   #
   # retrieve proto_node /usr/opt/SUNWrdc/lib/rdc_link_up file
   #
   &exec_cmd ( "$RCP $proto_ip:${remote_rdc_link_up} $local_rdc_link_up") ||
     &error ( "Unable to retrieve proto_node:$remote_rdc_link_up in $local_rdc_link_up");
	
   #
   # modify locally proto_node rdc_link_up file
   #
   &update_rdc_file( $local_rdc_link_up, $INTERVAL_ENTRY, $INTERVAL_VALUE);

   #
   # putback modified proto_node rdc_link_up file
   #
   &exec_cmd( "$RCP $local_rdc_link_up $proto_ip:${remote_rdc_link_up}" ) ||
     &error ( "Unable to putback $local_rdc_link_up on proto_node:${remote_rdc_link_up}") ;


   #
   # If this is the MEN2, update the mc_rms.conf and /etc/system file
   #
   if ( $proto_node_id eq $MEN2_ID ) {
     
     #
     # retrieve proto_node /usr/kernel/drv/mc_rms.conf file
     #
     &exec_cmd ( "$RCP $proto_ip:${remote_mc_rms} $local_mc_rms") ||
       &error ( "Unable to retrieve proto_node:$remote_mc_rms in $local_mc_rms");
	
     #
     # modify locally proto_node mc_rms.conf file
     #

     &update_rdc_file( $local_mc_rms, $SYSTEMID_ENTRY, $SYSTEMID_VALUE);

     #
     # putback modified proto_node mc_rms.conf file
     #
     &exec_cmd( "$RCP $local_mc_rms $proto_ip:${remote_mc_rms}" ) ||
       &error ( "Unable to putback $local_mc_rms on proto_node:${remote_mc_rms}") ;
	  
     #
     # retrieve proto_node /etc/system file
     #
     &exec_cmd ( "$RCP $proto_ip:$remote_system $local_system") ||
       &error ( "Unable to retrieve proto_node:$remote_system in $local_system");
	
     #
     # modify locally proto_node /etc/system
     #
     &update_rdc_file( $local_system, $RDC_RPC_TMOUT, $RDC_RPC_TMOUT_VALUE );

     #
     # putback modified proto_node /etc/system
     #
     &exec_cmd( "$RCP $local_system $proto_ip:${remote_system}" ) ||
       &error ( "Unable to putback $local_system on proto_node:${remote_system}") ;

   }	
	
   #
   # create a bitmaps slice for each replicated slice defined in RNFS_SLICE_EXPORTED
   # with the specific size given as a count of 512bytes block.
   #

   for my $i (0..$#SLICE_LIST) {
     if ($SLICE_LIST[$i]{bitmap} ne "") {
       # compute block count
	my $size = $SLICE_LIST[$i]{size} * 1024;
	my $bits = divrounded($size, 32) ;
	my $bytes = 512 + divrounded($bits, 8) ;
	my $block_count = divrounded($bytes, 512) ;
	# create the bitmap slice
	my $bitmap_index;
	# look for the raw name of the bitmap slice (because of SVM)
	slice_search(name, $SLICE_LIST[$i]{bitmap}, \$bitmap_index);
	&remote_exec ( "$proto_ip", "$RDD if=/dev/zero of=$SLICE_LIST[$bitmap_index]{raw} count=$block_count" ) || 
	  &error ( "Unable to remotely create bitmap slices $SLICE_LIST[$bitmap_index]{raw} (block count=$block_count)" );
      }
   }
	
   #
   # Add all the replicated slices in the file sv.cf locally
   #
   open (SVCF, ">$local_sv_cf") || &error("Unable to open $local_sv_cf");
   
   for my $i (0..$#SLICE_LIST) {
     if ($SLICE_LIST[$i]{bitmap} ne "") {
       print SVCF "$SLICE_LIST[$i]{raw}\tcache\n" ;
     }
   }
	
   close (SVCF);
   
   #
   # putback the created proto_node sv.cf file
   #
   &exec_cmd( "$RCP $local_sv_cf $proto_ip:${remote_sv_cf}" ) ||
     &error ( "Unable to putback $local_sv_cf on proto_node:${remote_sv_cf}");
   
   #
   # Add all the replicated slices in the file crfs.conf locally
   #
   open (CRFSCONF, ">$local_crfs_conf") || &error("Unable to open $local_crfs_conf");
   
   print CRFSCONF "statd_alternate_path = $SLICE_SUNWCGHA\n";
 
   for my $i (0..$#SLICE_LIST) {
     if ($SLICE_LIST[$i]{bitmap} ne "") {
       my $bitmap_index;
       # look for the raw name of the bitmap slice (because of SVM)
       slice_search(name, $SLICE_LIST[$i]{bitmap}, \$bitmap_index);
       print CRFSCONF "slice = $SLICE_LIST[$i]{raw} $SLICE_LIST[$bitmap_index]{raw} $SLICE_LIST[$i]{raw} $SLICE_LIST[$bitmap_index]{raw} 1\n" ;
     }
   }
   
   close (CRFSCONF);

   #	
   # putback the created proto_node crfs.conf file
   #
   &exec_cmd( "$RCP $local_crfs_conf $proto_ip:${remote_crfs_conf}" ) ||
     &error ( "Unable to putback $local_crfs_conf on proto_node:${remote_crfs_conf}");
   
   #
   # create the nhcrfs.conf file and copy it
   #
   open (NHCRFSCONF, ">$local_nhcrfs_conf") 
     || error("Unable to open $local_nhcrfs_conf");
   print NHCRFSCONF "CRFS_OPTIONS=\"\"\n";
   if ($USE_CGTP eq "YES") {
     print NHCRFSCONF "CRFS_NICCGTP=cgtp0\n" ;
     print NHCRFSCONF "CRFS_NIC0=$MEN_NIC0\n" ;
     print NHCRFSCONF "CRFS_NIC1=$MEN_NIC1\n" ;
     print NHCRFSCONF "CRFS_USE_CGTP=YES\n" ;
   } else {
     print NHCRFSCONF "CRFS_NIC0=$MEN_NIC0\n" ;
     print NHCRFSCONF "CRFS_USE_CGTP=NO\n" ;
   }
   close (NHCRFSCONF) ;
   &exec_cmd( "$RCP $local_nhcrfs_conf $proto_ip:${remote_nhcrfs_conf}" ) ||
     &error ( "Unable to putback $local_nhcrfs_conf on proto_node:${remote_nhcrfs_conf}");

   #
   # Add all the replicated and exported NFS slices in the file crfs.share locally
   # !!!!!! BUT smdiskless does not succeeds if /export is already shared  (not in the array)
   #
   if ($proto_node_id eq $MEN1_ID) {
     # create the exported directories on the main node
     for my $i (0..$#DIR_EXPORTED) {
       remote_exec ( "$proto_ip", "$RMKDIR -p $DIR_EXPORTED[$i]\n") ||
	 error ( "Unable to create directory $DIR_EXPORTED[$i] on MEN1");;
     }
     # create also the remote mounted directories
     for my $i (0..$#DIR_MOUNTED) {
       remote_exec ( "$proto_ip", "$RMKDIR -p $DIR_MOUNTED[$i]{remote}\n") ||
	 error ( "Unable to create directory $DIR_MOUNTED[$i]{remote} on MEN1");;
     }
     
   }

   # build the node name list
   my $node_string = "";
   for my $node (0..$#NODE_LIST) {
     my $node_id = $NODE_LIST[$node]{id};

     my $nicname = "CGTP" ;
     if ($USE_CGTP ne "YES") {
       $nicname = "NIC0" ;
     }

     if ($node_string eq "") {
       $node_string = build_node_name($node_id, $nicname);
     } else {
       $node_string .= ":" . build_node_name($node_id, $nicname) ;
     }
   }

   # define the exported directory ... as exported
   open (CRFSHARE, ">$local_crfs_share") || &error("Unable to open $local_crfs_share");

   for $i (0..$#DIR_EXPORTED) {
     print CRFSHARE "share -F nfs -o rw,root=$node_string $DIR_EXPORTED[$i]\n";
   }
   
   close (CRFSHARE);
   
   #
   # putback the created proto_node crfs.share file
   #
   &exec_cmd( "$RCP $local_crfs_share $proto_ip:${remote_crfs_share}" ) ||
     &error ( "Unable to putback $local_crfs_share on proto_node:${remote_crfs_share}");
   
   #
   # retrieve proto_node /etc/vfstab file
   #
   &exec_cmd ( "$RCP $proto_ip:$remote_vfstab $local_vfstab") ||
     &error ( "Unable to retrieve proto_node:$remote_vfstab in $local_vfstab");
	
   #
   # modify locally /etc/vfstab for each replicated slice
   # no mount at boot: managed by CRFS
   #
   for $i (0..$#SLICE_LIST) {
     if ($SLICE_LIST[$i]{bitmap} ne "") {
       update_vfstab_ufs( $local_vfstab, "$SLICE_LIST[$i]{raw}", "no" );
     }
   }
   
   #
   # define all mounted directories
   #
   for $i (0..$#DIR_MOUNTED) {
     # create the directory if it doesn't exist
     remote_exec ( "$proto_ip", "$RMKDIR -p $DIR_MOUNTED[$i]{local}" ) || 
       &error ( "Unable to remotely create directory $DIR_MOUNTED[$i]{local}" ) ;
     # add an entry in /etc/vfstab
     update_vfstab_nfs( $local_vfstab, "${master_ip}:$DIR_MOUNTED[$i]{remote}",
		       "-\t$DIR_MOUNTED[$i]{local}\tnfs\t-\tno\trw,hard,fg,intr,noac,proto=udp");
   }
		
   #
   # putback modified proto_node /etc/vfstab
   #
   &exec_cmd( "$RCP $local_vfstab $proto_ip:${remote_vfstab}" ) ||
     &error ( "Unable to putback $local_vfstab on proto_node:${remote_vfstab}") ;
}


##################################################################################
#
#                               This is the main
#
##################################################################################
 
init_and_read_arg($0,
		  [ ROOT_DIR, PACKAGE_SUBDIR, LOCAL_PACKAGES, NODE_ROLE, WORKING_DIR ],
		  []);

if ($RECOVERY ne "") {
  print_recovery_node("Reliable NFS (RNFS) clean-up") ;	
  remove_men_pkg($NODE_ID, $LOCAL_PACKAGES, "LOCAL") ;
} else {

  print_stage_node("Reliable NFS (RNFS) installation") ;	
  install_men_pkg($NODE_ID, $ROOT_DIR, $PACKAGE_SUBDIR, $LOCAL_PACKAGES, "LOCAL", "") ;

  print_action ("RNFS configuration") ;
  rnfs_configure ($NODE_ID , $SERVER_NODE);
}

# That's all folk!
exit(0);







