#!/usr/bin/perl
# ------------------------------------------------------------------------------
#   ident "@(#)nmen_cmm.pl 1.10 02/03/15 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}

#-------------------------------------------------------------------------------
#
# Management of /etc/services file
#
#-------------------------------------------------------------------------------
sub add_services {
   my ( $services_file, $service_input, $port_entry_input ) = @_ ;
   my $count=0;
   my $service;
   my $port_entry;
   my $garbage;
 
   $count = 0;   
     
   open (SERVICES, $services_file ) || &error("Unable to open $services_file");
   while ( <SERVICES> ) {
      
      chop();
      s/#.*//;
      ($service, $port_entry, $garbage) = split (/ /, $_) ;
      next unless (&defined($service));
      next unless (&defined($port_entry));
            
      if ( "$service" eq "$service_input" ) {
	 &error("Service \"$service\" does not match input \"$port_entry_input\" in $services_file") 
	 unless ( $port_entry eq $port_entry_input );
         $count++;
	 last;
      }
       	        
   }
   close (SERVICES);
   
   unless ( $count ) {
      open (SERVICES, ">>$services_file" ) || &error("Unable to update $services_file");
      print SERVICES "$service_input      $port_entry_input\n";
      close (SERVICES);
   }
   
}

#-------------------------------------------------------------------------------
#
# Configure CMM 
#
#-------------------------------------------------------------------------------
sub cmm_configure {

  my ($cluster,$subnet0,$subnet1,$proto_node_id,$server_node) = @_;

  my $proto_ip   ="${subnet0}.${cluster}.${subnet1}.${proto_node_id}";
   
  my $server_ip  ="${subnet0}.${cluster}.${subnet1}.${server_node}";

  my $node_name_template;
  my $node_name_nic0_template;
  my $node_name_cgtp_template;
  my $node_root;
		
  my $local_cmm_conf = "$WORKING_DIR/nhcmm.conf" ;
  my $remote_cmm_conf;
  
  my $local_nodename = "$WORKING_DIR/nodename" ;
  my $remote_nodename;
   
  for $node (0..$#NODE_LIST) {
	
    if ($node ne $MEN1 && $node ne $MEN2) {
       
      my $node_id = $NODE_LIST[$node]{id};
      $node_name_template = build_node_name($node_id, "") ;
      $node_name_nic0_template = build_node_name($node_id, "NIC0") ;
      $node_name_cgtp_template = build_node_name($node_id, "CGTP") ;

      print_subaction("configuring $node_name_template ...");

      $node_root = "/export/root/$node_name_template";
      $remote_cmm_conf = "$node_root/etc/opt/SUNWcgha/nhcmm.conf";		
      $remote_nodename = "$node_root/etc/nodename";
						
       #
       # Create nhcmm.conf file
       #
       open(USER_CMM_CONFIG, ">$local_cmm_conf") ||
	 &error("Unable to create $local_cmm_conf");
       
       print USER_CMM_CONFIG<<"FEO";
	
# CMM_SHARED_CONFIG_DIR contains location of shared config. file.
CMM_SHARED_CONFIG_DIR=/SUNWcgha/remote/etc
export CMM_SHARED_CONFIG_DIR

# CMM_IS_ELIGIBILITY contains node eligibility (i.e. capacity of becoming a master)
CMM_IS_ELIGIBLE=NO
export CMM_IS_ELIGIBLE

# CMM_DOMAIN_ID contains the cluster id
CMM_DOMAIN_ID=${cluster}
export CMM_DOMAIN_ID

# SET LOCALE 
NLSPATH=/opt/SUNWcgha/lib/locale/C/LC_MESSAGES/%N
export NLSPATH

FEO

      if ($USE_CGTP eq "YES") {
       print USER_CMM_CONFIG<<"FEO";
	
# CMM_NIC0 and CMM_NIC1 contain the two network interfaces for CGTP.
CMM_NIC0=$NODE_LIST[$node]{nic0}
CMM_NIC1=$NODE_LIST[$node]{nic1}
export CMM_NIC0
export CMM_NIC1

# CMM_USE_CGTP indicates if cgtp routing has to be enabled
CMM_USE_CGTP=YES
export CMM_USE_CGTP

FEO
      } else {
       print USER_CMM_CONFIG<<"FEO";
	
# CMM_NIC0 contains the network interfaces
CMM_NIC0=$NODE_LIST[$node]{nic0}
CMM_NIC1=unused
export CMM_NIC0
export CMM_NIC1

# CMM_USE_CGTP indicates if cgtp routing has to be enabled
CMM_USE_CGTP=NO
export CMM_USE_CGTP

FEO
      }

      close(USER_CMM_CONFIG);
	
    #
    # remote copy nhcmm.conf
    #
    &exec_cmd ("$RCP $local_cmm_conf $proto_ip:$remote_cmm_conf") ||
      &error ( "Unable to remote copy $local_cmm_conf in $proto_ip:$remote_cmm_conf");

			
  #
  # create nodename file
  #
  open(NODENAME, ">$local_nodename") ||
    &error("Unable to create $local_nodename");

  if ($USE_CGTP eq "YES") {
      print NODENAME "${node_name_cgtp_template}\n";
  } else {
      print NODENAME "${node_name_nic0_template}\n";
  }
  close(NODENAME);
		
  #
  # remote copy nodename
  #
  &exec_cmd ("$RCP $local_nodename $proto_ip:$remote_nodename") ||
    &error ( "Unable to remote copy $local_nodename in $proto_ip:$remote_nodename");
   
}
}						
	
}


##################################################################################
#
#                               This is the main
#
##################################################################################

init_and_read_arg($0,
		  [ ROOT_DIR, PACKAGE_SUBDIR, LOCAL_PACKAGES, NODE_ROLE, WORKING_DIR ],
		  []);

if ($RECOVERY ne "") {

  print_recovery_node("Cluster Membership Monitor (CMM) clean-up for diskless") ;
  remove_nmen_pkg ($NODE_ID, $LOCAL_PACKAGES);

} else {

  print_stage_node("Cluster Membership Monitor (CMM) installation for diskless") ;
  install_nmen_pkg ($NODE_ID, $ROOT_DIR, $PACKAGE_SUBDIR, $LOCAL_PACKAGES, "");
  
  print_action("CMM configuration") ;
  cmm_configure ($CLUSTER, $SUBNET0, $SUBNET1, $NODE_ID , $SERVER_NODE);

}

# That's all folk!
exit(0);

