#!/usr/bin/perl
# ------------------------------------------------------------------------------
#   ident "@(#)nmen_end.pl 1.13 02/03/15 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}

#-------------------------------------------------------------------------------
#
# update node reference in /etc/vfstab
#
# Replace all references to a node with logical address of master interface 
# typically for root partition of a diskless
#
#-------------------------------------------------------------------------------

sub update_vfstab_master {
  my ( $vfstab_file, $update_nfs_ip_address_input ) = @_ ;
  my $update_nfs_ip_address;
  my $update_nfs_value;
  my $line ;

  open (VFSTAB, $vfstab_file ) || &error("Unable to open $vfstab_file");
  open (VFSTAB_TMP, ">${vfstab_file}.tmp" ) || &error("Unable to open ${vfstab_file}.tmp");
  
  while ( <VFSTAB> ) {
    
    chop();
    $line = $_ ;
    s/#.*//;
    ($update_nfs_ip_address, $update_nfs_value) = split (/:/, $_) ;
    
    if ( "$update_nfs_value" ne "" ) {
      print VFSTAB_TMP "$update_nfs_ip_address_input:$update_nfs_value\n";
    } else {
      print VFSTAB_TMP "$line\n";
    }
    
    next;       	        
  }
  close (VFSTAB);
  close (VFSTAB_TMP);
  
  &exec_cmd ( "$MV ${vfstab_file}.tmp ${vfstab_file}") ||
    &error ( "Unable to move ${vfstab_file}.tmp ${vfstab_file}");  
}

#-------------------------------------------------------------------------------
#
# update mounting point in /etc/vfstab 
#
#  add a mounting point in vfstab
#
#-------------------------------------------------------------------------------

sub update_vfstab_nfs {
  my ( $vfstab_file, $update_nfs_input, $update_nfs_value_input ) = @_ ;
  my $update_nfs;
  my $update_nfs_value;
  my $garbage;
  my $count = 0;
  my $line ;
  
  open (VFSTAB, $vfstab_file ) || &error("Unable to open $vfstab_file");
  open (VFSTAB_TMP, ">${vfstab_file}.tmp" ) || &error("Unable to open ${vfstab_file}.tmp");
  
  while ( <VFSTAB> ) {
    
    chop();
    $line = $_ ;
    s/#.*//;
    ($update_nfs, $garbage) = split (/\t/, $_) ;
    
    if ( "$update_nfs" eq "$update_nfs_input" ) {
      print VFSTAB_TMP "$update_nfs\t$update_nfs_value_input\n";
      $count++;
    } else {
      print VFSTAB_TMP "$line\n";
    }
    next;       	        
  }
  close (VFSTAB);
  close (VFSTAB_TMP);
  
  unless ($count) {
    open (VFSTAB_TMP, ">>${vfstab_file}.tmp" ) || &error("Unable to update $vfstab_file");
    print VFSTAB_TMP "$update_nfs_input\t$update_nfs_value_input\n";
    close (VFSTAB_TMP);
  }
   
  &exec_cmd ( "$MV ${vfstab_file}.tmp ${vfstab_file}") ||
    &error ( "Unable to move ${vfstab_file}.tmp ${vfstab_file}");  
}



#-------------------------------------------------------------------------------
#
# VFSTAB end
#
#-------------------------------------------------------------------------------
sub vfstab_end {

  my ($cluster,$proto_node_id,$server_node) = @_;
  
  my $proto_ip   ="${SUBNET0}.${cluster}.${SUBNET1}.${proto_node_id}";
	
  my $server_ip  ="${SUBNET0}.${cluster}.${SUBNET1}.${server_node}";

  my $node_name_template;
  my $node_root;
	
  my $local_vfstab;
  my $remote_vfstab;

  my $master_nic0 = build_node_name($MASTER_ID, "NIC0") ;
  my $master_cgtp = build_node_name($MASTER_ID, "CGTP") ;

  print_action("Updating /etc/vfstab") ;
	
  for $node (0..$#NODE_LIST) {

    if ($node ne $MEN1 && $node ne $MEN2) {
      
      $node_name_template = build_node_name($NODE_LIST[$node]{id}, "");
      print_subaction("for $node_name_template") ;

      $node_root = "/export/root/$node_name_template";
      
      $local_vfstab = "$WORKING_DIR/vfstab_${node_name_template}";
      $remote_vfstab = "$node_root/etc/vfstab";
						
      #
      # retrieve diskless node /etc/vfstab file
      #
      &exec_cmd ( "$RCP $proto_ip:$remote_vfstab $local_vfstab") ||
	&error ( "Unable to retrieve $proto_node_id:$remote_vfstab in $local_vfstab");
			
      # 
      # update mounted ip address with master_ip
      #
      if ($USE_CGTP eq "YES") {
	&update_vfstab_master( $local_vfstab, $master_cgtp );
      } else {
	&update_vfstab_master( $local_vfstab, $master_nic0 );
      }

      #
      # add an entry for each remote directory
      for $i (0..$#DIR_MOUNTED) {
	# create the mounting point
	remote_exec( $proto_ip, "$RMKDIR -p $node_root/$DIR_MOUNTED[$i]{local}" ) ||
	  &error ( "Unable to create $node_root/$DIR_MOUNTED[$i]{local}" );
	my $master_name = $master_cgtp ;
	if ($USE_CGTP ne "YES") {
	  $master_name = $master_nic0 ;
	}
	update_vfstab_nfs($local_vfstab,"${master_name}:$DIR_MOUNTED[$i]{remote}","-\t$DIR_MOUNTED[$i]{local}\tnfs\t-\tyes\trw,hard,fg,intr,noac")
	}
      #
      # putback modified proto_node /etc/vfstab on diskless node branch
      #
      &exec_cmd( "$RCP $local_vfstab $proto_ip:${remote_vfstab}" ) ||
	&error ( "Unable to putback $local_vfstab on proto_node:${remote_vfstab}") ;
    }
    
  }

}

	
#-------------------------------------------------------------------------------
#
# remote the configured file
#
#-------------------------------------------------------------------------------
sub cmm_end {

  my ($cluster,$proto_node_id,$server_node) = @_;

  my $proto_ip   ="${SUBNET0}.${cluster}.${SUBNET1}.${proto_node_id}";
	
  my $server_ip  ="${SUBNET0}.${cluster}.${SUBNET1}.${server_node}";
	
  my $node_name_template;
  my $node_root;
  
  my $config_in_progress;

  print_action("Removing /etc/opt/SUNWcgha/not_configured file");
  
  for $node (0..$#NODE_LIST) {
	
    my $node_id = $NODE_LIST[$node]{id};
    $node_name_template = build_node_name($node_id, "");

    if ($node ne $MEN1 && $node ne $MEN2) {
      
      print_subaction("for $node_name_template") ;
      $node_root = "/export/root/$node_name_template";
      
      $config_in_progress = "$node_root/etc/opt/SUNWcgha/not_configured";				
      
      #
      # remove file /etc/opt/SUNWcgha/not_configured on diskless node
      #
      &remote_exec( "$proto_ip", "$RRM -f $config_in_progress" ) ||
	&error ( "Unable to remove $config_in_progress file" );	
      
    } else {
      # not removed previously if the install is optimized
      if ($ENV{OPTIMIZED_INSTALL} eq "YES") {

	print_subaction("for $node_name_template") ;
	my $node_ip = "${SUBNET0}.${cluster}.${SUBNET1}.${node_id}";
	$config_in_progress = "/etc/opt/SUNWcgha/not_configured";

	&remote_exec( "$node_ip", "$RRM -f $config_in_progress" ) ||
	  &error ( "Unable to remove $config_in_progress file" );	
      }
    }
  }
}


##################################################################################
#
#                               This is the main
#
##################################################################################
    
init_and_read_arg($0,
		  [ NODE_ROLE, WORKING_DIR ],
		  []);

print_stage_node("Foundation services post-installation for diskless") ;
vfstab_end ($CLUSTER, $NODE_ID, $SERVER_NODE);
cmm_end ($CLUSTER, $NODE_ID, $SERVER_NODE);

# That's all folk!
exit(0);







