#!/usr/bin/perl
# ------------------------------------------------------------------------------
#   ident "@(#)nmen_rbs.pl 1.21 02/04/11 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}

#-------------------------------------------------------------------------------
#
# special command execution without standard error device redirection
#
#-------------------------------------------------------------------------------

sub exec_cmd_local {

    my ($command) = @_;

    @result = qx{$command 2> /dev/null};
  
    # in case of error, display output and exit
    if ( $? ne 0 ) {
      $status = $? / 256 ;
      print_log("-----------------------------------------------------------------------------\n");
      print_log("Error when executing: $command (status=$status)\n") ;
      foreach $line (@result)
	{
	  print_log("  $line\n") ;
	}
      print_log("-----------------------------------------------------------------------------\n");
       exit 1;
    }

   # if it's OK, display if tracing is required

    foreach $line (@result)
      {
	&trace( 1, $line);
      }
  
    return 1;
}

#-------------------------------------------------------------------------------
#
# replace a entry in dhcpsvc.conf
#
#-------------------------------------------------------------------------------

sub replace_in_file {
   my ( $file, $update_entry_input, $update_value_input ) = @_ ;
   my $count=0;
   my $update_entry;
   my $update_value;
   my $garbage;
 
   my $count = 0;   
     
   open (FILE, $file ) || &error("Unable to open $file");
	open (FILE_TMP, ">${file}.tmp" ) || &error("Unable to open ${file}.tmp");
	
   while ( <FILE> ) {
      
      chop();
      ($update_entry, $update_value, $garbage) = split (/=/, $_) ;
            
      if ( "$update_entry" eq "$update_entry_input" ) {
			print FILE_TMP "${update_entry}=${update_value_input}\n";
			$count++;
		} else {
			print FILE_TMP "$_\n";
		}
		next;
       	        
   }
   close (FILE);
	close (FILE_TMP);
   
   unless ( $count ) {
      open (FILE_TMP, ">>${file}.tmp" ) || &error("Unable to update $file");
      print FILE_TMP "$update_entry_input=$update_value_input\n";
      close (FILE_TMP);
   }
	
	&exec_cmd ( "$MV ${file}.tmp ${file}") ||
	&error ( "Unable to move ${file}.tmp ${file}");
   
}

#-------------------------------------------------------------------------------
#
# update address entry in share lines to have:
#  -o rw=$name:$name_nic1:$name_cgtp0,root=$name:$name_nic1:$name_cgtp0
#
#-------------------------------------------------------------------------------
sub update_address_in_file {
   my ($file, $name, $name_nic0, $name_nic1, $name_cgtp0) = @_;

   my $count=0;
   my $before_root_entry;
   my $root_till_end_line;
   my $cgha_before_flag;
   my $cgha_after_flag;
   my $root_entry;
   my $new_root;
   my $new_entry;
   my $fs_Path;
   my $share_entry ="share -F nfs -o";
   my $acces_entry ="rw=${name}";

	# Update the rw= with the triplet diskless addresses
	# Update the root= with the triplet diskless addresses
	
   open (FILE, $file ) || &error("Unable to open $file");
	open (FILE_TMP, ">${file}.tmp" ) || &error("Unable to open ${file}.tmp");
	
   while ( <FILE> ) {
      
      chop();
      ($before_root_entry, $root_till_end_line) = split (/,/, $_) ;
            
      if ( "$before_root_entry" ne "${share_entry} ${acces_entry}" ) {
			 print FILE_TMP "$_\n";
      }
      else {
		($root_entry, $fs_path) = split(/ /, $root_till_end_line);
		if ($USE_CGTP eq "YES") {
		  $new_root="root=${name_nic0}:${name_nic1}:${name_cgtp0}";
		  $new_access="rw=${name_nic0}:${name_nic1}:${name_cgtp0}";
		} else {
		  $new_root="root=${name_nic0}";
		  $new_access="rw=${name_nic0}";
		}
                $new_entry="${share_entry} ${new_access},${new_root} ${fs_path}";
                print FILE_TMP "${new_entry}\n";
      }
      next; 	        
   }
   close (FILE);
	close (FILE_TMP);
	
	&exec_cmd ( "$MV ${file}.tmp ${file}") ||
   &error ( "Unable to move ${file}.tmp ${file}");
	
	# Remove the comments lines (specially those added by CRFS : CGHA START & CGHA END)
	
	open (FILE, $file ) || &error("Unable to open $file");
	open (FILE_TMP, ">${file}.tmp" ) || &error("Unable to open ${file}.tmp");
	
   while ( <FILE> ) {
	      chop();
		  ($cgha_before_flag, $cgha_after_flag) = split (/#/, $_);
		  if ( "$cgha_after_flag" eq "" ) {
			 		print FILE_TMP "$_\n";
		  }
		  next;
	}
	close (FILE);
	close (FILE_TMP);
	
	&exec_cmd ( "$MV ${file}.tmp ${file}") ||
   &error ( "Unable to move ${file}.tmp ${file}");

}


#-------------------------------------------------------------------------------
#
# Management of /etc/inet/dhcpsvc.conf file
#
#-------------------------------------------------------------------------------
sub update_dhcpsvcconf {

   my ($proto_node_id,$server_node) = @_;

   my $proto_name = build_node_name($proto_node_id, "");
   my $proto_ip   ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${proto_node_id}";
   my $men2_ip ="${SUBNET0}.${CLUSTER}.${SUBNET1}.$MEN2_ID";
	
   my $server_ip  ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${server_node}";
	
   my $local_dhcpsvc_file = "$WORKING_DIR/dhcpsvc_${proto_name}";
   my $remote_dhcpsvc_file = "/etc/inet/dhcpsvc.conf";

   my $dhcp_container_dir="/SUNWcgha/remote/var/dhcp";

   if ( "$WORKAROUND_RBSCONFIG_SCRIPT" eq "FALSE" ) {
     #
     # retrieve  dhcpsvc.conf file
     #
     &exec_cmd ("$RCP $proto_ip:$remote_dhcpsvc_file $local_dhcpsvc_file") ||
       &error ( "Unable to remote copy $proto_ip:$remote_dhcpsvc_file in $local_dhcpsvc_file");

     #
     # locally update dhcpsvc.conf file
     #
     open (DHCPSVC, ">>$local_dhcpsvc_file" ) || &error("Unable to open $local_dhcpsvc_file");
     if ($USE_CGTP eq "YES") {
       print  DHCPSVC "INTERFACES=$MEN_NIC0,$MEN_NIC1\n";
     } else {
       print  DHCPSVC "INTERFACES=$MEN_NIC0\n";
     }
     print  DHCPSVC "OFFER_CACHE_TIMEOUT=30\n";
     close (DHCPSVC);

     &replace_in_file($local_dhcpsvc_file, "RESOURCE", "SUNWnhrbs") ||
       &error( "Unable to update RESOURCE line in $local_dhcpsvc_file" );

     &replace_in_file($local_dhcpsvc_file, "PATH", "$dhcp_container_dir") ||
       &error( "Unable to update PATH line in $local_dhcpsvc_file" );

     #
     # remote copy dhcpsvc.conf file
     #
     &exec_cmd ("$RCP $local_dhcpsvc_file $proto_ip:$remote_dhcpsvc_file") ||
       &error ( "Unable to remote copy $local_dhcpsvc_file in $proto_ip:$remote_dhcpsvc_file");
	
     #
     # update MEN2 with this file
     #
     &exec_cmd ( "$RCP $local_dhcpsvc_file  $men2_ip:$remote_dhcpsvc_file") ||
       &error ( "Unable to remote copy $local_dhcpsvc_file in $men2_ip:$remote_dhcpsvc_file");				
   }
}

#------------------------------------------------------------------------------------
# 
# Workaround to have on the second master eligible node the same environment for TFTP
# as the one on the first master eligible node:
#  - /tftpboot with the diskless hexadecimal address files linked to the inetboot file
#  - the service tftp is configured in /etc/inet/inetd.conf
# without this workaround diskless cannot be booted from this node when is elected as
# the master after a switch over/ fail over
#
#------------------------------------------------------------------------------------
sub rbs_vicemaster_workaround {

  my ($master_ip, $vice_master_ip) = @_;

  my $local_inetdconf = "$WORKING_DIR/inetdconf.tmp";

  # Copy /etc/inet/inetd.conf from first master eligible node to the second
  # master eligible node to have the needed entry of tftp when this latest node
  # becomes the master
  #
  exec_cmd("$RCP ${master_ip}:/etc/inet/inetd.conf $local_inetdconf") ||
    error ( "Unable to copy ${master_ip}:/etc/inet/inetd.conf on $local_inetdconf");
  exec_cmd("$RCP $local_inetdconf ${vice_master_ip}:/etc/inet/inetd.conf") ||
    error ( "Unable to copy $local_inetdconf to ${vice_master_ip}:/etc/inet/inetd.conf ");
    
  #
  # Create /tftpboot on the second master eligible node and populate it
  # with the files got from the /tftpboot on the first master eligible node
  # 
  &remote_exec("$vice_master_ip", "$RRM -fr /tftpboot") ;
  &remote_exec("$vice_master_ip", "$RMKDIR /tftpboot") ||
    &error( "Unable to create remotely /tftpboot on ${vice_master_ip}" );
  
#  &remote_exec("$master_ip", "$RFIND /tftpboot | $RCPIO -omB | $RRSH ${vice_master_ip} $RCPIO -idumvB");

  # can not use standard exec_cmd command because of invalid output redirection
  exec_cmd_local("$RSH -n $master_ip \"$RFIND /tftpboot | $RCPIO -omB\" > $WORKING_DIR/cpio.tmp") ||
    error("creating cpio dump of /tftpboot from men1");
  exec_cmd("$CAT $WORKING_DIR/cpio.tmp | $RSH $vice_master_ip $RCPIO -idumvB") ||
    error("creating /tftpboot from cpio dump on men2");
    
}

#-------------------------------------------------------------------------------
#
# Management of /etc/opt/SUNWcgha/crfs.share file
#
#-------------------------------------------------------------------------------
sub rbs_update_crfs_share {

  my ($first_men_id,$second_men_id, $server_node) = @_;

  my $server_ip           ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${server_node}";

  my $first_men_name      = build_node_name($first_men_id, "");
  my $first_men_ip        ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${first_men_id}";

  my $second_men_name     = build_node_name($second_men_id, "");
  my $second_men_ip       ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${second_men_id}";

  
  my $node_name_template;
  my $node_name_if0_template;
  my $node_name_if1_template;
  my $node_name_cgtp0_template;

  my $remote_crfs_share_file  ="/etc/opt/SUNWcgha/crfs.share";
  my $local_crfs_share_file   ="$WORKING_DIR/crfs_share_${first_men_name}";

  #
  # retrieve remote file
  #
  &exec_cmd ("$RCP $first_men_ip:$remote_crfs_share_file $local_crfs_share_file") ||
    &error ( "Unable to remote copy $first_men_ip:$remote_crfs_share_file in $local_crfs_share_file");
	
  #
  # locally update the file with all diskless triplet addresses.
  #

  for my $node (0..$#NODE_LIST) {
    if ($node ne $MEN1 && $node ne $MEN2) {
      my $node_id=$NODE_LIST[$node]{id};
      $node_name_template = build_node_name($node_id, "");
      $node_name_if0_template = build_node_name($node_id, "NIC0");
      $node_name_if1_template = build_node_name($node_id, "NIC1");
      $node_name_cgtp0_template = build_node_name($node_id, "CGTP");

      &update_address_in_file($local_crfs_share_file, "${node_name_template}", "${node_name_if0_template}",
			      "${node_name_if1_template}", "${node_name_cgtp0_template}") ||
				&error( "Unable to update diskless $node_name_template in $local_crfs_share_file" );
    }
  }
   
  #
  # remote copy the crfs_share file on first MEN node
  #
  &exec_cmd ("$RCP $local_crfs_share_file $first_men_ip:$remote_crfs_share_file") ||
    &error ( "Unable to remote copy $local_crfs_share_file in $first_men_ip:$remote_crfs_share_file");
	
  #
  # update second MEN with this file
  #
  &exec_cmd ( "$RCP $local_crfs_share_file $second_men_ip:$remote_crfs_share_file") ||
    &error ( "Unable to remote copy $local_crfs_share_file in $second_men_ip:$remote_crfs_share_file");
}
#-------------------------------------------------------------------------------
#
# build Client ID
#
#-------------------------------------------------------------------------------

sub build_cid {

  my ($eth) = @_ ;

  # build the Client ID from the MAC address
  my $cid = "01" ;
  foreach $string (split(':',$eth)) { 
    if (length($string) == 0) {
      $cid .= "00" ;
    } elsif (length($string) == 1) {
      $cid .= "0$string" ;
    } else {
      $cid .= $string;
    }
  }

  # be sure that letters are uppercase
  $cid =~ tr/a-z/A-Z/;

  return $cid;

}
#-------------------------------------------------------------------------------
#
# Configure RBS 
#
#-------------------------------------------------------------------------------
sub rbs_configure {

  my ($proto_node_id,$server_node) = @_;

  my $proto_ip   ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${proto_node_id}";
	
  my $server_ip  ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${server_node}";
	
  my $net_if0 ="${SUBNET0}.${CLUSTER}.${SUBNET1}.0";
  my $net_if1 ="${SUBNET0}.${CLUSTER}.${SUBNET2}.0";
	
  my $broadcast_if0 ="${SUBNET0}.${CLUSTER}.${SUBNET1}.255";
  my $broadcast_if1 ="${SUBNET0}.${CLUSTER}.${SUBNET2}.255";
	
  my $master_ip_if0   ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${MASTER_ID}";
  my $master_ip_if1 ="${SUBNET0}.${CLUSTER}.${SUBNET2}.${MASTER_ID}";
  my $master_ip_cgtp0 ="${SUBNET0}.${CLUSTER}.${SUBNET3}.${MASTER_ID}";
	
  my $netmask ="255.255.255.0";
	
#  my $dhcp_container_dir="/SUNWcgha/remote/var/dhcp";
  my $dhcp_container_dir="/SUNWcgha/local/export/data/var/dhcp";
  my $defaultdomain = "/etc/defaultdomain" ;
	
  my $node_ip_if0_template;
  my $node_ip_if1_template;
	
  my $node_name_template;
  my $node_name_if0_template;
  my $node_name_if1_template;
	
  my $cmd0;
  my $cmd1;
  my $dhcmd0;
  my $dhcmd1;
  my $dhcmd2;
  my $dhcmd3;
  my $dhcmd4;
  my $dhcmd5;
  my $dhcmd6;
  
  my $platform;	
	
  my $vice_master_ip="${SUBNET0}.${CLUSTER}.${SUBNET1}.${MEN2_ID}";


  #
  # Even if CGTP is not used, both NIC0 and NIC1 need to be configured in order to create both parameters file
  # if not, Reliable Boot server will never reply to the dhcp request because of an error due to a missing file
  #


  #
  # remotely stop dhcp
  #
  &remote_exec ( "$proto_ip", "/etc/rc2.d/HA.K21dhcp stop") ||
    &error( " Unable to stop dhcp daemon\n" );

  if ( "$WORKAROUND_RBSCONFIG_SCRIPT" eq "FALSE" ) {
    
    #
    # remotely execute dhcpconfig
    #
    &remote_exec ( "$proto_ip", "$RDHCPCONFIG -D -r SUNWfiles -p $dhcp_container_dir -n" ) ||
      &error( " Unable to execute $RDHCPCONFIG -D -r SUNWfiles -p $dhcp_container_dir -n" );
    
    # 
    # remotely create DHCP containers by invoking pntadm -C
    #
    &remote_exec ( "$proto_ip", "$RPNTADM -C 	$net_if0" ) ||
      &error( " Unable to execute $RPNTADM -C 	$net_if0" );
    &remote_exec ( "$proto_ip", "$RPNTADM -C 	$net_if1" ) ||
      &error( " Unable to execute $RPNTADM -C 	$net_if1" );

    #
    # remotely add DHCP network entries in DHCP containers by invoking pntadm -C
    #
    for $node (0..$#NODE_LIST) {
      my $node_id = $NODE_LIST[$node]{id};
      $node_ip_if0_template ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${node_id}";
      $node_ip_if1_template = "${SUBNET0}.${CLUSTER}.${SUBNET2}.${node_id}";

      if ($node ne $MEN1 && $node ne $MEN2) {

	# the following two commands generate warning
	# so do not display them to avoid the customer to be confused

	my $subcmd0 = "-f PERMANENT" ;
	my $subcmd1 = "-f PERMANENT" ;
	if ($DHCP_MODE eq "STATIC") {
	  my $cid = build_cid($NODE_LIST[$node]{eth0}) ;
	  $subcmd0 = "-f PERMANENT+MANUAL -i $cid" ;
	  $cid = build_cid($NODE_LIST[$node]{eth1}) ;
	  $subcmd1 = "-f PERMANENT+MANUAL -i $cid" ;
	}

	$node_name_if0_template = build_node_name($node_id, "NIC0");
	$cmd0 = "$RPNTADM -A $node_ip_if0_template $subcmd0 -h $node_name_if0_template -m $node_ip_if0_template $net_if0";
	&remote_exec_ignore ( "$proto_ip", $cmd0) ||
	  &trace(1," Cannot remotely execute $cmd0 but we don't care..." );

	$node_name_if1_template = build_node_name($node_id, "NIC1");;
	$cmd1 = "$RPNTADM -A $node_ip_if1_template $subcmd1 -h $node_name_if1_template -m $node_ip_if1_template $net_if1";
	&remote_exec_ignore ( "$proto_ip", $cmd1) ||
	  &trace(1," Cannot remotely execute $cmd1 but we don't care" );
      }
    }

    $platform = "";
    for $i (0..$#VENDOR_TYPE) {
      if ($platform eq "") {
	$platform = $VENDOR_TYPE[$i] ;
      } else {
	$platform .= " " . $VENDOR_TYPE[$i];
      }
    }
    $platform = "\"Vendor=$platform\"" ;

    $dhcmd0 ="$RDHTADM -A -s SbootFIL -d \'$platform,7,ASCII,1,0\'";
    $dhcmd1 ="$RDHTADM -A -s SswapPTH -d \'$platform,6,ASCII,1,0\'";
    $dhcmd2 ="$RDHTADM -A -s SswapIP4 -d \'$platform,5,IP,1,0\'";
    $dhcmd3 ="$RDHTADM -A -s SrootPTH -d \'$platform,4,ASCII,1,0\'";
    $dhcmd4 ="$RDHTADM -A -s SrootNM -d \'$platform,3,ASCII,1,0\'";
    $dhcmd5 ="$RDHTADM -A -s SrootIP4 -d \'$platform,2,IP,1,1\'";
    $dhcmd6 ="$RDHTADM -A -s SrootOpt -d \'$platform,1,ASCII,1,0\'";

    &remote_exec_bis ( "$proto_ip", $dhcmd0) ||
      &error( " Cannot remotely execute $dhcmd0" );
    &remote_exec_bis ( "$proto_ip", $dhcmd1) ||
      &error( " Cannot remotely execute $dhcmd1" );	
    &remote_exec_bis ( "$proto_ip", $dhcmd2) ||
      &error( " Cannot remotely execute $dhcmd2" );				
    &remote_exec_bis ( "$proto_ip", $dhcmd3) ||
      &error( " Cannot remotely execute $dhcmd3" );
    &remote_exec_bis ( "$proto_ip", $dhcmd4) ||
      &error( " Cannot remotely execute $dhcmd4" );
    &remote_exec_bis ( "$proto_ip", $dhcmd5) ||
      &error( " Cannot remotely execute $dhcmd5" );
    &remote_exec_bis ( "$proto_ip", $dhcmd6) ||
      &error( " Cannot remotely execute $dhcmd6" );

    #
    # remotely add	macros to dhcptab container
    #
    $dhcmd7 ="$RDHTADM -A -m $net_if0 -d \':Broadcst=$broadcast_if0:MTU=1500:Router=$master_ip_if0:\'";
    &remote_exec ( "$proto_ip", $dhcmd7) ||
      &error( " Cannot remotely execute $dhcmd7" );

    $dhcmd8 ="$RDHTADM -A -m $net_if1 -d \':Broadcst=$broadcast_if1:MTU=1500:Router=$master_ip_if1:\'";
    &remote_exec ( "$proto_ip", $dhcmd8) ||
      &error( " Cannot remotely execute $dhcmd8" );
    
    my $master_ip = $master_ip_cgtp0 ;
    if ($USE_CGTP ne "YES") {
      $master_ip = $master_ip_if0;
    }

    $dhcmd9 ="$RDHTADM -A -m Common -d \':Include=Locale:BootSrvA=$master_ip:SrootIP4=$master_ip:BootSrvN=$master_ip:SrootNM=$master_ip:BootFile=inetboot.$ARCH.Solaris_8:Subnet=$netmask:\'";	
    &remote_exec ( "$proto_ip", $dhcmd9) ||
      &error( " Cannot remotely execute $dhcmd9" );	

    for $node (0..$#NODE_LIST) {
      my $node_id = $NODE_LIST[$node]{id} ;
      $node_ip_if0_template = "${SUBNET0}.${CLUSTER}.${SUBNET1}.${node_id}";
      $node_ip_if1_template = "${SUBNET0}.${CLUSTER}.${SUBNET2}.${node_id}";

      if ($node ne $MEN1 && $node ne $MEN2) {
	$node_name_template = build_node_name($node_id, "");

	$node_name_if0_template = build_node_name($node_id, "NIC0");
	$cmd10 = "$RDHTADM -A -m $node_ip_if0_template -d \':Include=Common:Include=$net_if0:SrootPTH=\"\/export\/root\/$node_name_template\":SswapPTH=\"\/export\/swap\/$node_name_template\":\'";
	&remote_exec ( "$proto_ip", $cmd10) ||
	  &error( " Cannot remotely execute $cmd10" );

	$node_name_if1_template = build_node_name($node_id, "NIC1");
	$cmd11 = "$RDHTADM -A -m $node_ip_if1_template -d \':Include=Common:Include=$net_if1:SrootPTH=\"\/export\/root\/$node_name_template\":SswapPTH=\"\/export\/swap\/$node_name_template\":\'";
	&remote_exec ( "$proto_ip", $cmd11) ||
	  &error( " Cannot remotely execute $cmd11" );
      }
    }

    #
    # specific modification for RBS
    # rename containers SUNWfiles_x_x_x_x in SUNWnhrbs_x_x_x_x
    #
    &remote_exec( "$proto_ip", "$RMV $dhcp_container_dir/SUNWfiles1_${SUBNET0}_${CLUSTER}_${SUBNET1}_0 $dhcp_container_dir/SUNWnhrbs1_${SUBNET0}_${CLUSTER}_${SUBNET1}_0") ||
      &error( "Unable to rename container dhcp files" );
    
    &remote_exec( "$proto_ip", "$RMV $dhcp_container_dir/SUNWfiles1_${SUBNET0}_${CLUSTER}_${SUBNET2}_0 $dhcp_container_dir/SUNWnhrbs1_${SUBNET0}_${CLUSTER}_${SUBNET2}_0") ||
      &error( "Unable to rename container dhcp files" );
    
    &remote_exec( "$proto_ip", "$RMV $dhcp_container_dir/SUNWfiles1_dhcptab $dhcp_container_dir/SUNWnhrbs1_dhcptab ") ||
      &error( "Unable to rename container dhcp files" );
	

    #
    # apply the workaround for boot server on second master eligible node when elected as the master
    #

    if ( "$WORKAROUND_RBS_VICEMASTER_TFTP" eq "TRUE" ) {
      &trace(1,"rbs_vicemaster_workaround START");
      &rbs_vicemaster_workaround ($proto_ip,$vice_master_ip);
      &trace(1,"rbs_vicemaster_workaround END");
    }
    
  }

  # create /etc/defaultdomain on both MEN
  remote_exec("$proto_ip",       "$RECHO \"$DOMAIN\" > $defaultdomain") ;
  remote_exec("$vice_master_ip", "$RECHO \"$DOMAIN\" > $defaultdomain") ;
}

#-------------------------------------------------------------------------------
#
# Undo RBS 
#
#-------------------------------------------------------------------------------

sub rbs_cleanup
  {
    my $node_ip   ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${NODE_ID}";
    remote_exec_ignore("$node_ip", "$RDHCPCONFIG -U -f -x -h") ;
  }

##################################################################################
#
#                               This is the main
#
##################################################################################

init_and_read_arg($0,
		  [ ROOT_DIR, PACKAGE_SUBDIR, LOCAL_PACKAGES, NODE_ROLE, WORKING_DIR ],
		  []);

if ($RECOVERY ne "") {

  if ($NODE_ID eq $MEN1_ID) {
    print_recovery_node("Reliable Boot Server (RBS) clean-up") ;
    rbs_cleanup() ;
  }
  
} else {
  
  print_stage_node("Reliable Boot Server (RBS) installation") ;
  install_men_pkg($NODE_ID, $ROOT_DIR, $PACKAGE_SUBDIR, $LOCAL_PACKAGES, "LOCAL", "") ;

  
  if ( $NODE_ID eq $MEN1_ID ) {

    print_action("RBS configuration") ;

    rbs_configure ($NODE_ID , $SERVER_NODE);
    update_dhcpsvcconf($NODE_ID , $SERVER_NODE);
    
    # check the workaround  $WORKAROUND_RBS_MEN_CRFS_SHARE
    # if set to TRUE, we activate the workaround function rbs_update_crfs_share
    
    if ("${WORKAROUND_RBS_MEN_CRFS_SHARE}" eq "TRUE") {
      #
      # update crfs_share on both eligible master nodes to have the diskless 
      # entries (root and swap)
      # with r/w access and root permissions set for the triplet of addresses
      # on subnet1, subnet2, and subnet3 (cgtp network).
      #
      rbs_update_crfs_share($MEN1_ID,$MEN2_ID, $SERVER_NODE);
    }
  }
}

# That's all folk!
exit(0);







